function [a, m, cm, lp] = EM(x, N, iter, m, cm)
%
% Expectation Maximazation algorithm for the estimation of Gaussian Mixture
% Models. Following Bilmes' paper (A Gentle Tutorial of the EM Algorithm...).
% This algorithm is quite fast but a good initialization should be external.
% The covariance matrices are full matrices.
%
% ----INPUT PARAMETERS----
%   x: dxL matrix wtih each column being a sample of dimension d
%   N: the number of Gaussian components
%   iter: the number of iterations
%   m: initial mean values (dxN matrix)
%   cm: initial covariance matrices (dxd*N matrix)
%
% ----OUTPUT PARAMETERS----
%   a: Nx1 vector with the probability of each mixture
%   m: dxN matrix with the mean values
%   cm: dxd*N matrix with covariance matirces
%   lp: likelihood
%
% (c) Yannis Pantazis, 2004, csd, uoc
%     mail: pantazis@csd.uoc.gr


[dimx,lenx] = size(x);

% INITIALIZATION
a = ones(1, N)/N;
if(nargin<5)
   cm = repmat(eye(dimx), 1, N);
end
if(nargin<4)
  m = x(:,1:N);
end

% NEW VALUES
an = zeros(1, N);
mn = zeros(dimx, N);
cmn = repmat(eye(dimx), 1, N);

flagstop=0;
lp = zeros(1,iter);
liter=1;
while(flagstop==0 && liter<iter)
    % normalization factor
    nf = ppe(x, a, m, cm);
    lp(liter) = sum(nf);
    
    % test if the improvement of the likelihood is satisfactory. If it
    % isn't stop the estimation otherwise continue.
    if(liter>1) 
        thres=(lp(liter)-lp(liter-1))/lp(liter);
        if(thres<2*10^(-5))
            flagstop=1;
        end
    end
    fprintf(1,'Iter: %d Likelihood : %f\n', liter , lp(liter));
    
    % for each gaussian component compute the new parameters
    for i = 1:N
        vv = gau(x, m(:,i), cm(:, (i-1)*dimx+1 : i*dimx))./nf;
        an(i) = a(i)*sum(vv, 2) / lenx;
      
        mn(:,i) = a(i)*sum(x.*repmat(vv, dimx, 1), 2) / (lenx*an(i));
  
        for k = 1:dimx
            for kk = 1:dimx
                cmn(k, (i-1)*dimx+kk) = a(i)*sum(vv .* (x(k,:)-mn(k,i)).*(x(kk,:)-mn(kk,i))) / (lenx*an(i));
            end
        end
    end
    % update
    a = an;
    m = mn;
    cm = cmn;
    liter=liter+1;
end
lp = lp(1:liter-1);
